/*
 * University of Illinois Open Source License
 * Copyright 2011 Luthey-Schulten Group,
 * All rights reserved.
 * 
 * Developed by: Luthey-Schulten Group
 * 			     University of Illinois at Urbana-Champaign
 * 			     http://www.scs.uiuc.edu/~schulten
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the Software), to deal with 
 * the Software without restriction, including without limitation the rights to 
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
 * of the Software, and to permit persons to whom the Software is furnished to 
 * do so, subject to the following conditions:
 * 
 * - Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimers.
 * 
 * - Redistributions in binary form must reproduce the above copyright notice, 
 * this list of conditions and the following disclaimers in the documentation 
 * and/or other materials provided with the distribution.
 * 
 * - Neither the names of the Luthey-Schulten Group, University of Illinois at
 * Urbana-Champaign, nor the names of its contributors may be used to endorse or
 * promote products derived from this Software without specific prior written
 * permission.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL 
 * THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR 
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
 * OTHER DEALINGS WITH THE SOFTWARE.
 *
 * Author(s): Elijah Roberts
 */

#ifndef LM_ME_MESOLVERFACTORY_H_
#define LM_ME_MESOLVERFACTORY_H_

#include <string>

using std::string;

namespace lm {
namespace me {

class MESolver;

/// @class MESolverFactory
/// @brief A factory object used to create Master Equation simulation instances
class MESolverFactory
{
public:
    /// @brief Create the factory
    MESolverFactory();
    
    /// @brief Set the type of solver the factory makes
    /// @param solver The name of the solver from the command line
    void setSolver(string solver) throw(lm::InvalidArgException);
    /// @brief Tell if solver needs a reaction model
    bool needsReactionModel() throw(lm::Exception);
    /// @brief Tell if solver needs a diffusion model
    bool needsDiffusionModel() throw(lm::Exception);
    
    /// @brief Get an instance of a MESolver object for running a replicate
    /// @return The new solver instance
    MESolver * instantiate() throw(lm::Exception);

private:
    string solver;
};

}
}

#endif
