#ifndef __ZDIVMULTIGPUMAPPER__
#define __ZDIVMULTIGPUMAPPER__

#include "MultiGPUMapper.h"

#define NUM_SHADOWS 2
#define Z_SHADOW_TOP 0
#define Z_SHADOW_BOTTOM 1

#define MCLKR_MAX_NBUFFERS 2

struct neighbor_buffer
{
	char *buffer[MCLKR_MAX_NBUFFERS];
	size_t size;
};

struct gpu_info
{
	dim3 global_pos;		// Position in the global lattice that this gpu is associated with
	dim3 global_dim;		// Dimensions in the global lattice that this is responsible for

	dim3 local_dim;			// Local GPU dimensions, including all padding
	dim3 local_authority;	// Position in local GPU that begins authoritative data

	int3 dependent_pos;		// Position in global lattice where we can get local_dim data
							// NOTE: Can be negative for periodic conditions

	dim3 overlap_dim;
	dim3 overlap_send[NUM_SHADOWS];
	dim3 overlap_recv[NUM_SHADOWS];
	int neighbor[NUM_SHADOWS];
	neighbor_buffer *read_buffer[NUM_SHADOWS];
	neighbor_buffer *write_buffer[NUM_SHADOWS];

//	void *current_state;

	int lb_chunks;			// Number of lattice chunks assigned for load balancing
	int lb_max_chunks;
	float lb_imbalance;
};

	
class ZDivMultiGPUMapper : public MultiGPUMapper
{
	private:
		gpu_info *info;
		bool periodic_z;
	
	protected:
		virtual void initialize();
		//void copy_from_neighbor(int gpu, int neighbor, void *dptr, int key);
		//void copy_to_neighbor(int gpu, int neighbor, int key);
		void stage_in_real(int gpu, void *dptr, void *hptr, unsigned int element_size);
		void stage_out_real(int gpu, void *hptr, void *dptr);

		bool determine_load_balance();
		int alter_chunks(int gpu, int count);

	public:
		ZDivMultiGPUMapper(dim3, size_t, int, int, int, int* gl=NULL, bool pz=false, int pages=1);
		ZDivMultiGPUMapper(int x, int y, int z, size_t, int, int, int, int* gl=NULL, bool pz=false, int pages=1);
		~ZDivMultiGPUMapper();
	
		virtual void initialize_gpu(int gpu);
		dim3 get_global_dim(int gpu);
		dim3 get_local_dim(int gpu);
		int3 get_global_offset(int gpu);
		size_t get_local_size(int gpu);
		size_t get_authority_size(int gpu);
		ssize_t get_global_input_offset(int gpu);
		size_t get_global_output_offset(int gpu);
		size_t get_authority_offset(int gpu);

		void stage_in(int gpu, void *dptr, void *hptr);
		void stage_in_sites(int gpu, void *dptr, void *hptr);
		void stage_out(int gpu, void *hptr, void *dptr);

		void publish_state(int gpu, void *dptr, int key);
		void refresh(int gpu, void *dptr, int key);
		virtual void schedule_send(int gpu, void *dptr, int timestamp, int neighbor, cudaStream_t stream);
		virtual void schedule_recv(int gpu, void *dptr, int timestamp, int neighbor, cudaStream_t stream);

		int map_index_to_gpu(size_t index);

};

#endif
